ONOS-3655 - Adding username and password support into the device key subsystem.

Change-Id: I196984479126ae3776093a0bded4b3c820a3eab8
diff --git a/core/api/src/main/java/org/onosproject/net/AnnotationKeys.java b/core/api/src/main/java/org/onosproject/net/AnnotationKeys.java
index 84a439d..0c7bb86 100644
--- a/core/api/src/main/java/org/onosproject/net/AnnotationKeys.java
+++ b/core/api/src/main/java/org/onosproject/net/AnnotationKeys.java
@@ -135,6 +135,16 @@
     public static final String MANAGEMENT_ADDRESS = "managementAddress";
 
     /**
+     * Annotation key for the username.
+     */
+    public static final String USERNAME = "username";
+
+    /**
+     * Annotation key for the password.
+     */
+    public static final String PASSWORD = "password";
+
+    /**
      * Returns the value annotated object for the specified annotation key.
      * The annotated value is expected to be String that can be parsed as double.
      * If parsing fails, the returned value will be 1.0.
diff --git a/incubator/api/src/main/java/org/onosproject/incubator/net/key/CommunityName.java b/incubator/api/src/main/java/org/onosproject/incubator/net/key/CommunityName.java
index 24b93a0..02a9ad3 100644
--- a/incubator/api/src/main/java/org/onosproject/incubator/net/key/CommunityName.java
+++ b/incubator/api/src/main/java/org/onosproject/incubator/net/key/CommunityName.java
@@ -19,7 +19,7 @@
 /**
  * Representation of an SNMP community name authentication token.
  */
-public final class CommunityName {
+public final class CommunityName extends DeviceKey {
     private final String name;
 
     /**
diff --git a/incubator/api/src/main/java/org/onosproject/incubator/net/key/DeviceKey.java b/incubator/api/src/main/java/org/onosproject/incubator/net/key/DeviceKey.java
index 51d042b..a9f0f5f 100644
--- a/incubator/api/src/main/java/org/onosproject/incubator/net/key/DeviceKey.java
+++ b/incubator/api/src/main/java/org/onosproject/incubator/net/key/DeviceKey.java
@@ -125,4 +125,36 @@
         String name = annotations().value(AnnotationKeys.NAME);
         return CommunityName.communityName(name);
     }
+
+    /**
+     * Method to create a device key of type USERNAME_PASSWORD.
+     *
+     * @param id    device key identifier
+     * @param label optional label for this device key
+     * @param username username for this device key
+     * @param password password for this device key
+     * @return device key
+     */
+    public static DeviceKey createDeviceKeyUsingUsernamePassword(DeviceKeyId id, String label,
+                                                                 String username, String password) {
+        DefaultAnnotations annotations = builder().set(AnnotationKeys.USERNAME, username)
+                .set(AnnotationKeys.PASSWORD, password).build();
+
+        return new DeviceKey(id, label, Type.USERNAME_PASSWORD, annotations);
+    }
+
+    /**
+     * Returns a username and password object from the device key.
+     *
+     * @return username and password
+     */
+    public UsernamePassword asUsernamePassword() {
+
+        // Validate that the device key is of type USERNAME_PASSWORD.
+        checkState(this.type == Type.USERNAME_PASSWORD, "This device key is not a USERNAME_PASSWORD type.");
+
+        String username = annotations().value(AnnotationKeys.USERNAME);
+        String password = annotations().value(AnnotationKeys.PASSWORD);
+        return UsernamePassword.usernamePassword(username, password);
+    }
 }
diff --git a/incubator/api/src/main/java/org/onosproject/incubator/net/key/UsernamePassword.java b/incubator/api/src/main/java/org/onosproject/incubator/net/key/UsernamePassword.java
new file mode 100644
index 0000000..e3ca1fb
--- /dev/null
+++ b/incubator/api/src/main/java/org/onosproject/incubator/net/key/UsernamePassword.java
@@ -0,0 +1,65 @@
+/*
+ * Copyright 2016 Open Networking Laboratory
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
+package org.onosproject.incubator.net.key;
+
+/**
+ * Representation of a username and password.
+ */
+public final class UsernamePassword extends DeviceKey {
+    private final String username;
+    private final String password;
+
+    /**
+     * Private constructor for a username and password.
+     *
+     * @param username user's name
+     * @param password user's password
+     */
+    private UsernamePassword(String username, String password) {
+        this.username = username;
+        this.password = password;
+    }
+
+    /**
+     * Static method to construct a username / password.
+     *
+     * @param username user's name
+     * @param password user's password
+     * @return username and password
+     */
+    static UsernamePassword usernamePassword(String username, String password) {
+        return new UsernamePassword(username, password);
+    }
+
+    /**
+     * Returns the username.
+     *
+     * @return username
+     */
+    public String username() {
+        return username;
+    }
+
+    /**
+     * Returns the password.
+     *
+     * @return password
+     */
+    public String password() {
+        return password;
+    }
+}
diff --git a/incubator/api/src/test/java/org/onosproject/incubator/net/key/UsernamePasswordTest.java b/incubator/api/src/test/java/org/onosproject/incubator/net/key/UsernamePasswordTest.java
new file mode 100644
index 0000000..242a263
--- /dev/null
+++ b/incubator/api/src/test/java/org/onosproject/incubator/net/key/UsernamePasswordTest.java
@@ -0,0 +1,64 @@
+/*
+ * Copyright 2016 Open Networking Laboratory
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
+package org.onosproject.incubator.net.key;
+
+import org.junit.Test;
+
+import static org.junit.Assert.*;
+import static org.onlab.junit.ImmutableClassChecker.assertThatClassIsImmutable;
+
+/**
+ * Test class for UsernamePassword.
+ */
+public class UsernamePasswordTest {
+    final String username = "username";
+    final String password = "password";
+
+    /**
+     * Checks that the UsernamePassword class is immutable.
+     */
+    @Test
+    public void testImmutability() {
+        assertThatClassIsImmutable(UsernamePassword.class);
+    }
+
+    /**
+     * Checks the construction of a username / password object with null
+     * values passed into it.
+     */
+    @Test
+    public void testUsernamePasswordNull() {
+        UsernamePassword usernamePassword = UsernamePassword.usernamePassword(null, null);
+
+        assertNotNull("The UsernamePassword should not be null.", usernamePassword);
+        assertNull("The username should be null.", usernamePassword.username());
+        assertNull("The password should be null.", usernamePassword.password());
+    }
+
+    /**
+     * Checks the construction of a username / password object with non-null
+     * values passed into it.
+     */
+    @Test
+    public void testUsernamePassword() {
+        UsernamePassword usernamePassword = UsernamePassword.usernamePassword(username, password);
+
+        assertNotNull("The usernamePassword should not be null.", usernamePassword);
+        assertEquals("The username should match the expected value.", username, usernamePassword.username());
+        assertEquals("The password should match the expected value.", password, usernamePassword.password());
+    }
+}
diff --git a/incubator/api/src/test/java/org/onosproject/incubator/net/key/DeviceKeyTest.java b/incubator/api/src/test/java/org/onosproject/incubator/net/key/DeviceKeyTest.java
index 5a5699b..52e5e88 100644
--- a/incubator/api/src/test/java/org/onosproject/incubator/net/key/DeviceKeyTest.java
+++ b/incubator/api/src/test/java/org/onosproject/incubator/net/key/DeviceKeyTest.java
@@ -29,9 +29,11 @@
     final String deviceKeyIdValue = "DeviceKeyId1";
     final String deviceKeyLabel = "DeviceKeyLabel";
     final String deviceKeySnmpName = "DeviceKeySnmpName";
+    final String deviceKeyUsername = "DeviceKeyUsername";
+    final String deviceKeyPassword = "DeviceKeyPassword";
 
     /**
      * Checks the construction of a device key name with a null
      * value passed into it. This will throw a NullPointerException.
      */
     @Test(expected = NullPointerException.class)
@@ -40,7 +42,7 @@
     }
 
     /**
      * Checks the construction of a device key name with non-null
      * values passed into it.
      */
     @Test
@@ -58,4 +60,63 @@
         assertNotNull("The communityName should not be null.", communityName);
         assertEquals("The name should match as expected", deviceKeySnmpName, communityName.name());
     }
+
+    /**
+     * Checks the invalid conversion a device key of type COMMUNITY_NAME to
+     * a username / password object.
+     */
+    @Test(expected = IllegalStateException.class)
+    public void testInvalidConversionToAsUsernamePassword() {
+        DeviceKeyId deviceKeyId = DeviceKeyId.deviceKeyId(deviceKeyIdValue);
+
+        DeviceKey deviceKey = DeviceKey.createDeviceKeyUsingCommunityName(deviceKeyId,
+                                                                          deviceKeyLabel, deviceKeySnmpName);
+        // Attempting to convert this device key to a username / password object
+        // should throw and IllegalStateException.
+        UsernamePassword usernamePassword = deviceKey.asUsernamePassword();
+    }
+
+    /**
+     * Checks the construction of a username / password device key with a null
+     * value passed into it. This will throw a NullPointerException.
+     */
+    @Test(expected = NullPointerException.class)
+    public void testCreateDeviceKeyUsingUsernamePasswordWithNull() {
+        DeviceKey deviceKey = DeviceKey.createDeviceKeyUsingUsernamePassword(null, null, null, null);
+    }
+
+    /**
+     * Checks the construction of a username and password device key name with non-null
+     * values passed into it.
+     */
+    @Test
+    public void testCreateDeviceKeyUsingUsernamePassword() {
+        DeviceKeyId deviceKeyId = DeviceKeyId.deviceKeyId(deviceKeyIdValue);
+
+        DeviceKey deviceKey = DeviceKey.createDeviceKeyUsingUsernamePassword(deviceKeyId, deviceKeyLabel,
+                                                                             deviceKeyUsername, deviceKeyPassword);
+        assertNotNull("The deviceKey should not be null.", deviceKey);
+        assertEquals("The deviceKeyId should match as expected", deviceKeyId, deviceKey.deviceKeyId());
+        assertEquals("The label should match as expected", deviceKeyLabel, deviceKey.label());
+        assertEquals("The type should match as expected", DeviceKey.Type.USERNAME_PASSWORD, deviceKey.type());
+
+        UsernamePassword usernamePassword = deviceKey.asUsernamePassword();
+        assertNotNull("The usernamePassword should not be null.", usernamePassword);
+        assertEquals("The username should match as expected", deviceKeyUsername, usernamePassword.username());
+        assertEquals("The password should match as expected", deviceKeyPassword, usernamePassword.password());
+    }
+
+    /**
+     * Checks the invalid conversion a device key of type USERNAME_PASSWORD to
+     * a community name object.
+     */
+    @Test(expected = IllegalStateException.class)
+    public void testInvalidConversionToAsCommunityName() {
+        DeviceKeyId deviceKeyId = DeviceKeyId.deviceKeyId(deviceKeyIdValue);
+
+        DeviceKey deviceKey = DeviceKey.createDeviceKeyUsingUsernamePassword(deviceKeyId, deviceKeyLabel,
+                                                                             deviceKeyUsername, deviceKeyPassword);
+        // Attempting to convert this device key to a community name should throw and IllegalStateException.
+        CommunityName communityName = deviceKey.asCommunityName();
+    }
 }
@@ -29,9 +29,11 @@
 
     /**
-     * Checks the construction of a device key name with a null
+     * Checks the construction of a community name device key with a null
      * value passed into it. This will throw a NullPointerException.
      */
@@ -40,7 +42,7 @@
 
     /**
-     * Checks the construction of a device key name with non-null
+     * Checks the construction of a community name device key name with non-null
      * values passed into it.
      */